@@ -1,4 +1,5 @@
 import pydas.exceptions
+import time
 
 def reauth(fn):
     """
@@ -9,17 +10,27 @@ def reauth(fn):
     @wraps(fn)
     def wrapper(*args, **kw):
         try:
-            return fn(*args, **kw)
-        except pydas.exceptions.PydasException:
+            retVal = fn(*args, **kw)
+            return retVal
+        except pydas.exceptions.PydasException as detail:
+            print "Caught PydasException: ", detail
+            print "Waiting 30 seconds, then retrying request"
+
+            # wait 30 seconds before retrying
+            time.sleep(30) 
+
             # renew the token
             # get the instance of the CoreDriver and set it as "that"
             that = args[0]
             token = that.login_with_api_key(that.__class__.email,
                                             that.__class__.apikey)
+
             # now fix up the arguments of the original call to use the renewed token
             argsList = list(args)
             argsList[2]['token'] = token
             args = tuple(argsList)
+
             # try the api call again
-            return fn(*args, **kw)
+            retVal = fn(*args, **kw)
+            return retVal
     return wrapper
